@@ -46,7 +46,7 @@ class po2tmx:
 
     def convertfiles(
         self, inputfile, tmxfile, sourcelanguage="en", targetlanguage=None, comment=None
-    ):
+    ) -> None:
         """Converts a .po file (possibly many) to TMX file."""
         inputstore = po.pofile(inputfile)
         for inunit in inputstore.units:
@@ -83,7 +83,7 @@ def convertpo(
     sourcelanguage="en",
     targetlanguage=None,
     comment=None,
-):
+) -> int:
     """Reads in stdin using fromfileclass, converts using convertorclass, writes to stdout."""
     convertor = po2tmx()
     convertor.convertfiles(
@@ -93,7 +93,7 @@ def convertpo(
 
 
 class tmxmultifile:
-    def __init__(self, filename, mode=None):
+    def __init__(self, filename, mode=None) -> None:
         """Initialises tmxmultifile from a seekable inputfile or writable outputfile."""
         self.filename = filename
         if mode is None:
@@ -107,7 +107,7 @@ class tmxmultifile:
     def openoutputfile(self, subfile):
         """Returns a pseudo-file object for the given subfile."""
 
-        def onclose(contents):
+        def onclose(contents) -> None:
             pass
 
         outputfile = wStringIO.CatchStringOutput(onclose)
@@ -117,7 +117,7 @@ class tmxmultifile:
 
 
 class TmxOptionParser(convert.ArchiveConvertOptionParser):
-    def recursiveprocess(self, options):
+    def recursiveprocess(self, options) -> None:
         if not options.targetlanguage:
             raise ValueError("You must specify the target language")
         super().recursiveprocess(options)
@@ -126,7 +126,7 @@ class TmxOptionParser(convert.ArchiveConvertOptionParser):
             self.outputarchive.tmxfile.serialize(self.output)
 
 
-def main(argv=None):
+def main(argv=None) -> None:
     formats = {"po": ("tmx", convertpo), ("po", "tmx"): ("tmx", convertpo)}
     archiveformats = {(None, "output"): tmxmultifile, (None, "template"): tmxmultifile}
     parser = TmxOptionParser(
