@@ -1,4 +1,3 @@
-#
 # Copyright 2002-2006 Zuza Software Foundation
 #
 # This file is part of translate.
@@ -21,25 +20,29 @@ String processing utilities for extracting strings with various kinds of
 delimiters.
 """
 
+from __future__ import annotations
+
 import html.entities
 import logging
 import re
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from collections.abc import Iterable
+
+logger = logging.getLogger(__name__)
 
 
-def find_all(searchin, substr):
+def find_all(searching: str, substr: str) -> Iterable[int]:
     """
-    Returns a list of locations where substr occurs in searchin locations
+    Returns a list of locations where substr occurs in searching locations
     are not allowed to overlap.
     """
     location = 0
-    locations = []
     substr_len = len(substr)
-    while location != -1:
-        location = searchin.find(substr, location)
-        if location != -1:
-            locations.append(location)
-            location += substr_len
-    return locations
+    while (location := searching.find(substr, location)) != -1:
+        yield location
+        location += substr_len
 
 
 def extract(
@@ -55,14 +58,14 @@ def extract(
     enteredonce = False
     lenstart = len(startdelim)
     lenend = len(enddelim)
-    startdelim_places = find_all(source, startdelim)
+    startdelim_places = list(find_all(source, startdelim))
     if startdelim == enddelim:
-        enddelim_places = startdelim_places[:]
+        enddelim_places = startdelim_places.copy()
     else:
-        enddelim_places = find_all(source, enddelim)
+        enddelim_places = list(find_all(source, enddelim))
     if escape is not None:
         lenescape = len(escape)
-        escape_places = find_all(source, escape)
+        escape_places = list(find_all(source, escape))
         # Filter escaped escapes
         true_escape = False
         true_escape_places = []
@@ -130,15 +133,15 @@ def extractwithoutquotes(
     enteredonce = False
     lenstart = len(startdelim)
     lenend = len(enddelim)
-    startdelim_places = find_all(source, startdelim)
+    startdelim_places = list(find_all(source, startdelim))
     if startdelim == enddelim:
-        enddelim_places = startdelim_places[:]
+        enddelim_places = startdelim_places.copy()
     else:
-        enddelim_places = find_all(source, enddelim)
+        enddelim_places = list(find_all(source, enddelim))
     # hell slow because it is called far too often
     if escape is not None:
         lenescape = len(escape)
-        escape_places = find_all(source, escape)
+        escape_places = list(find_all(source, escape))
         # filter escaped escapes
         true_escape = False
         true_escape_places = []
@@ -236,23 +239,22 @@ def extractwithoutquotes(
 
 
 # TODO: investigate if ord is needed
-def _encode_entity_char(char, codepoint2name):
+def _encode_entity_char(char: str, codepoint2name: dict[str, str]) -> str:
     charnum = ord(char)
     if charnum in codepoint2name:
-        return "&%s;" % codepoint2name[charnum]
+        return f"&{codepoint2name[charnum]};"
     return char
 
 
-def entityencode(source, codepoint2name):
+def entityencode(source: str, codepoint2name: dict[str, str]) -> str:
     """
     Encode ``source`` using entities from ``codepoint2name``.
 
     :param unicode source: Source string to encode
     :param codepoint2name: Dictionary mapping code points to entity names
            (without the the leading ``&`` or the trailing ``;``)
-    :type codepoint2name: :meth:`dict`
     """
-    output = ""
+    output = []
     inentity = False
     for char in source:
         if char == "&":
@@ -261,27 +263,31 @@ def entityencode(source, codepoint2name):
             continue
         if inentity:
             if char == ";":
-                output += "&" + possibleentity + ";"
+                output.append(f"&{possibleentity};")
                 inentity = False
             elif char == " ":
-                output += _encode_entity_char("&", codepoint2name) + entityencode(
-                    possibleentity + char, codepoint2name
+                output.extend(
+                    (
+                        _encode_entity_char("&", codepoint2name),
+                        entityencode(possibleentity + char, codepoint2name),
+                    )
                 )
                 inentity = False
             else:
                 possibleentity += char
         else:
-            output += _encode_entity_char(char, codepoint2name)
+            output.append(_encode_entity_char(char, codepoint2name))
     if inentity:
         # Handle nonentities at end of string.
-        output += _encode_entity_char("&", codepoint2name) + entityencode(
-            possibleentity, codepoint2name
+        output.append(
+            _encode_entity_char("&", codepoint2name)
+            + entityencode(possibleentity, codepoint2name)
         )
 
-    return output
+    return "".join(output)
 
 
-def _has_entity_end(source):
+def _has_entity_end(source: str) -> bool:
     for char in source:
         if char == ";":
             return True
@@ -290,16 +296,15 @@ def _has_entity_end(source):
     return False
 
 
-def entitydecode(source, name2codepoint):
+def entitydecode(source: str, name2codepoint: dict[str, str]) -> str:
     """
     Decode ``source`` using entities from ``name2codepoint``.
 
     :param unicode source: Source string to decode
     :param name2codepoint: Dictionary mapping entity names (without the
            the leading ``&`` or the trailing ``;``) to code points
-    :type name2codepoint: :meth:`dict`
     """
-    output = ""
+    output = []
     inentity = False
     for i, char in enumerate(source):
         char = source[i]
@@ -312,24 +317,24 @@ def entitydecode(source, name2codepoint):
                 if len(possibleentity) > 0 and possibleentity in name2codepoint:
                     entchar = chr(name2codepoint[possibleentity])
                     if entchar == "&" and _has_entity_end(source[i + 1 :]):
-                        output += "&" + possibleentity + ";"
+                        output.append(f"&{possibleentity};")
                     else:
-                        output += entchar
+                        output.append(entchar)
                     inentity = False
                 else:
-                    output += "&" + possibleentity + ";"
+                    output.append(f"&{possibleentity};")
                     inentity = False
             elif char == " ":
-                output += "&" + possibleentity + char
+                output.append(f"&{possibleentity}{char}")
                 inentity = False
             else:
                 possibleentity += char
         else:
-            output += char
+            output.append(char)
     if inentity:
         # Handle nonentities at end of string.
-        output += "&" + possibleentity
-    return output
+        output.append(f"&{possibleentity}")
+    return "".join(output)
 
 
 def htmlentityencode(source):
@@ -341,7 +346,7 @@ def htmlentityencode(source):
     return entityencode(source, html.entities.codepoint2name)
 
 
-def htmlentitydecode(source):
+def htmlentitydecode(source: str) -> str:
     """
     Decode source using HTML entities e.g. ``&copy;`` -> ©.
 
@@ -350,40 +355,83 @@ def htmlentitydecode(source):
     return entitydecode(source, html.entities.name2codepoint)
 
 
-def javapropertiesencode(source):
-    """
+def javapropertiesencode(source: str, encoding: str | None = None) -> str:
+    r"""
     Encodes source in the escaped-unicode encoding used by Java
     .properties files.
+
+    :param source: The string to encode
+    :param encoding: The target encoding (default: iso-8859-1).
+                     Only characters that cannot be represented in this
+                     encoding will be escaped as \uXXXX.
     """
-    output = ""
+    if encoding is None:
+        encoding = "iso-8859-1"
+
+    output = []
     if source and source[0] == " ":
-        output = "\\"
+        output.append("\\")
+
+    # Determine character validation function based on encoding
+    # Fast path for common encodings using ordinal checks (no exceptions)
+    if encoding in {"ascii", "us-ascii"}:
+
+        def is_valid_char(charnum):
+            return 0 <= charnum < 128
+    elif encoding in {"iso-8859-1", "latin-1", "latin1"}:
+
+        def is_valid_char(charnum):
+            return 0 <= charnum <= 255
+    else:
+        # For other encodings, try the full string first for performance
+        try:
+            source.encode(encoding)
+            # All characters are valid, use a function that always returns True
+
+            def is_valid_char(charnum) -> bool:
+                return True
+        except (UnicodeEncodeError, LookupError):
+            # Some characters can't be encoded, need per-character check
+            def is_valid_char(charnum) -> None:
+                return None  # Signal to use try/except
+
+    # Process each character with the appropriate validation
     for char in source:
         charnum = ord(char)
         if char in controlchars:
-            output += controlchars[char]
-        elif 0 <= charnum < 128:
-            output += str(char)
+            output.append(controlchars[char])
         else:
-            output += "\\u%04X" % charnum
-    return output
+            valid = is_valid_char(charnum)
+            if valid is None:
+                # Need to test encoding for this character
+                try:
+                    char.encode(encoding)
+                    output.append(str(char))
+                except (UnicodeEncodeError, LookupError):
+                    output.append(f"\\u{charnum:04X}")
+            elif valid:
+                output.append(str(char))
+            else:
+                output.append(f"\\u{charnum:04X}")
+
+    return "".join(output)
 
 
-def java_utf8_properties_encode(source):
+def java_utf8_properties_encode(source: str) -> str:
     """
     Encodes source in the escaped-unicode encoding used by java utf-8
     .properties files.
     """
-    output = ""
+    output = []
     for char in source:
         if char in controlchars:
-            output += controlchars[char]
+            output.append(controlchars[char])
         else:
-            output += char
-    return output
+            output.append(char)
+    return "".join(output)
 
 
-def xwiki_properties_encode(source, encoding):
+def xwiki_properties_encode(source: str, encoding: str) -> str:
     if re.search(r"\{[0-9]+\}", source):
         source = source.replace("'", "''")
     if encoding == "utf-8":
@@ -391,24 +439,17 @@ def xwiki_properties_encode(source, encoding):
     return javapropertiesencode(source)
 
 
-def escapespace(char):
+def escapespace(char: str) -> str:
     assert len(char) == 1
     if char.isspace():
-        return "\\u%04X" % ord(char)
+        return f"\\u{ord(char):04X}"
     return char
 
 
-def mozillaescapemarginspaces(source):
+def mozillaescapemarginspaces(source: str) -> str:
     """Escape leading and trailing spaces for Mozilla .properties files."""
     if not source:
         return ""
-
-    if len(source) == 1 and source.isspace():
-        # FIXME: This is hack for people using white-space to mark empty
-        # Mozilla strings translated, drop this once we have better way to
-        # handle this in Pootle.
-        return ""
-
     if len(source) == 1:
         return escapespace(source)
     return escapespace(source[0]) + source[1:-1] + escapespace(source[-1])
@@ -427,23 +468,50 @@ propertyescapes = {
 }
 
 controlchars = {
-    # the reverse of the above...
-    "\\": "\\\\",
-    "\f": "\\f",
-    "\n": "\\n",
-    "\r": "\\r",
-    "\t": "\\t",
+    # complement of the above with all control chars
+    "\\": r"\\",
+    "\x00": r"\u0000",
+    "\x01": r"\u0001",
+    "\x02": r"\u0002",
+    "\x03": r"\u0003",
+    "\x04": r"\u0004",
+    "\x05": r"\u0005",
+    "\x06": r"\u0006",
+    "\x07": r"\u0007",
+    "\x08": r"\u0008",
+    "\t": r"\t",
+    "\n": r"\n",
+    "\x0b": r"\u000b",
+    "\f": r"\f",
+    "\r": r"\r",
+    "\x0e": r"\u000e",
+    "\x0f": r"\u000f",
+    "\x10": r"\u0010",
+    "\x11": r"\u0011",
+    "\x12": r"\u0012",
+    "\x13": r"\u0013",
+    "\x14": r"\u0014",
+    "\x15": r"\u0015",
+    "\x16": r"\u0016",
+    "\x17": r"\u0017",
+    "\x18": r"\u0018",
+    "\x19": r"\u0019",
+    "\x1a": r"\u001a",
+    "\x1b": r"\u001b",
+    "\x1c": r"\u001c",
+    "\x1d": r"\u001d",
+    "\x1e": r"\u001e",
+    "\x1f": r"\u001f",
 }
+controlchars_trans = str.maketrans(controlchars)
 
 
-def escapecontrols(source):
+def escapecontrols(source: str) -> str:
     """Escape control characters in the given string."""
-    for key, value in controlchars.items():
-        source = source.replace(key, value)
-    return source
+    return source.translate(controlchars_trans)
 
 
-def propertiesdecode(source):
+def propertiesdecode(source: str) -> str:
     """
     Decodes source from the escaped-unicode encoding used by .properties
     files.
@@ -454,7 +522,7 @@ def propertiesdecode(source):
     don't want to we reimplemented the algorithm from Python Objects/unicode.c
     in Python and modify it to retain escaped control characters.
     """
-    output = ""
+    output = []
     s = 0
 
     def unichr2(i):
@@ -468,19 +536,19 @@ def propertiesdecode(source):
             # we just return the character, unescaped
             # if people want to escape them they can use escapecontrols
             return chr(i)
-        return "\\u%04x" % i
+        return f"\\u{i:04x}"
 
     while s < len(source):
         c = source[s]
         if c != "\\":
-            output += c
+            output.append(c)
             s += 1
             continue
         s += 1
         if s >= len(source):
             # this is an escape at the end of the line, which implies
             # a continuation..., return the escape to inform the parser
-            output += c
+            output.append(c)
             continue
         c = source[s]
         s += 1
@@ -488,7 +556,7 @@ def propertiesdecode(source):
             pass
         # propertyescapes lookups
         elif c in propertyescapes:
-            output += propertyescapes[c]
+            output.append(propertyescapes[c])
         # \uXXXX escapes
         # \UXXXX escapes
         elif c in "uU":
@@ -509,46 +577,49 @@ def propertiesdecode(source):
                     digits = digit
                     break
             s += digits
-            output += unichr2(x)
+            output.append(unichr2(x))
         elif c == "N":
             if source[s] != "{":
-                logging.warning("Invalid named unicode escape: no { after \\N")
-                output += "\\" + c
+                logger.warning("Invalid named unicode escape: no { after \\N")
+                output.append(f"\\{c}")
                 continue
             s += 1
             e = source.find("}", s)
             if e == -1:
-                logging.warning("Invalid named unicode escape: no } after \\N{")
-                output += "\\" + c
+                logger.warning("Invalid named unicode escape: no } after \\N{")
+                output.append(f"\\{c}")
                 continue
-            import unicodedata
+            # pylint: disable-next=import-outside-toplevel
+            import unicodedata  # noqa: PLC0415
 
             name = source[s:e]
-            output += unicodedata.lookup(name)
+            output.append(unicodedata.lookup(name))
             s = e + 1
         else:
-            output += c  # Drop any \ that we don't specifically handle
-    return output
+            output.append(c)  # Drop any \ that we don't specifically handle
+    return "".join(output)
 
 
-def xwiki_properties_decode(source):
+def xwiki_properties_decode(source: str) -> str:
     if re.search(r"\{[0-9]+\}", source):
         source = source.replace("''", "'")
     return propertiesdecode(source)
 
 
-def findend(string, substring):
+def findend(string: str, substring: str) -> int:
     s = string.find(substring)
     if s != -1:
         s += len(substring)
     return s
 
 
-def rstripeol(string):
+def rstripeol(string: str) -> str:
     return string.rstrip("\r\n")
 
 
-def stripcomment(comment, startstring="<!--", endstring="-->"):
+def stripcomment(
+    comment: str, startstring: str = "<!--", endstring: str = "-->"
+) -> str:
     cstart = comment.find(startstring)
     if cstart == -1:
         cstart = 0
@@ -558,5 +629,7 @@ def stripcomment(comment, startstring="<!--", endstring="-->"):
     return comment[cstart:cend].strip()
 
 
-def unstripcomment(comment, startstring="<!-- ", endstring=" -->\n"):
+def unstripcomment(
+    comment: str, startstring: str = "<!-- ", endstring: str = " -->\n"
+) -> str:
     return startstring + comment.strip() + endstring
