@@ -24,6 +24,7 @@ from __future__ import annotations
 import copy
 import os
 import re
+from typing import ClassVar, overload
 from xml.parsers.expat import XML_PARAM_ENTITY_PARSING_NEVER, ParserCreate
 
 from lxml import etree
@@ -35,21 +36,12 @@ from translate.storage import base, lisa
 WHITESPACE = {" ", "\n", "\t"}  # Whitespace that we collapse.
 ESCAPE_NEWLINE = {"n", "N"}
 ESCAPE_TAB = {"t", "T"}
-ESCAPE_PLAIN = {" ", '"', "'", "@", "?"}
+ESCAPE_PLAIN = {" ", '"', "'", "@", "?", "\\"}
 MULTIWHITESPACE = re.compile(r"[ \n\t]{2}(?!\\n)")
 QUOTED_STRING = re.compile(r'((?<!\\)"(?:\\"|[^"])*(?<!\\)")')
 UNICODE_ESCAPE = re.compile(r"\\u([a-fA-F0-9]{4})")
 CHAR_ESCAPE = re.compile(r"\\(.)")
-
-ESCAPE_TRANSLATE = str.maketrans(
-    {
-        "\\": "\\\\",
-        "\n": "\\n",
-        "\t": "\\t",
-        "'": "\\'",
-        '"': '\\"',
-    }
-)
+WHITESPACE_RE = re.compile(r"\s+")
 
 
 class DecodingXMLParser:
@@ -62,21 +54,22 @@ class DecodingXMLParser:
     """
 
     EMIT_DEPTH = 1
-    FOREGIN_DTD = True
+    FOREIGN_DTD = True
 
-    def __init__(self, text: str):
+    def __init__(self, text: str, *, escape_all: bool = True) -> None:
         self.text = text.encode("utf-8")
-        self.output = []
-        self.emit_start = None
-        self.decoded_emit = None
+        self.output: list[str] = []
+        self.emit_start: int | None = None
         self.character_data = False
         self.raw_string = True
         self.in_string = False
+        self.inside_quoted = False
         self.do_cleanup = False
         self.depth = 0
+        self.escape_all = escape_all
         self.parser = parser = ParserCreate()
-        if self.FOREGIN_DTD:
-            parser.UseForeignDTD(self.FOREGIN_DTD)
+        if self.FOREIGN_DTD:
+            parser.UseForeignDTD(self.FOREIGN_DTD)
         parser.SetParamEntityParsing(XML_PARAM_ENTITY_PARSING_NEVER)
         parser.StartElementHandler = self.StartElementHandler
         parser.EndElementHandler = self.EndElementHandler
@@ -91,21 +84,44 @@ class DecodingXMLParser:
         # Convert from hex to character
         return chr(int(escaped, 16))
 
-    @staticmethod
-    def decode_escapes(match: re.Match) -> str:
+    def decode_escapes(self, match: re.Match) -> str:
         escaped = match.group(1)
         if escaped in ESCAPE_NEWLINE:
             return "\n"  # an actual newline
         if escaped in ESCAPE_TAB:
             return "\t"  # an actual tab
+        if escaped == "\\":
+            return "\\"
+        # Compose Multiplatform Resources handles escaping only for few chars
+        if not self.escape_all:
+            return rf"\{escaped}"
         if escaped in ESCAPE_PLAIN:
             # Plain string to escape
             return escaped
         # All others, remove, like Android does as well.
         return ""
 
-    @classmethod
-    def process_string(cls, content: str) -> tuple[str, bool, bool]:
+    def strip_part(self, text: str) -> tuple[str, bool]:
+        # Detect leading quote
+        if text.startswith('"'):
+            text = text[1:]
+            self.inside_quoted ^= True
+
+        # Check if whitespace should be stripped
+        strip_whitespace = not self.inside_quoted
+
+        # Detect trailing quote
+        if text.endswith('"') and not text.endswith('\\"'):
+            text = text[:-1]
+            self.inside_quoted ^= True
+
+        # Consolidate whitespace if needed
+        if strip_whitespace:
+            text = WHITESPACE_RE.sub(" ", text)
+
+        return text, strip_whitespace
+
+    def process_string(self, content: str) -> tuple[str, bool, bool]:
         # Skip processing for a blank string
         if not content:
             return "", False, False
@@ -114,27 +130,31 @@ class DecodingXMLParser:
         parts = [part for part in QUOTED_STRING.split(content) if part]
 
         # Detect possible cleanups
-        cleanup_start = not parts[0].startswith('"')
-        cleanup_end = not parts[-1].startswith('"')
+        cleanup_start = False
+        cleanup_end = False
 
         # Collapse whitespace
-        stripped_parts = [
-            re.sub(r"\s+", " ", part) if not part.startswith('"') else part[1:-1]
-            for part in parts
-        ]
+        stripped_parts: list[str] = []
+        for i, part in enumerate(parts):
+            stripped, was_stripped = self.strip_part(part)
+            if i == 0:
+                cleanup_start = was_stripped
+            cleanup_end = was_stripped and not self.inside_quoted
+            stripped_parts.append(stripped)
+
         # Merge this back to a single string
         text = "".join(stripped_parts)
 
         # Handle unicode escapes
-        unescaped_text = UNICODE_ESCAPE.sub(cls.decode_unicode_escapes, text)
+        unescaped_text = UNICODE_ESCAPE.sub(self.decode_unicode_escapes, text)
         # Handle other escapes
-        unescaped_text = CHAR_ESCAPE.sub(cls.decode_escapes, unescaped_text)
+        unescaped_text = CHAR_ESCAPE.sub(self.decode_escapes, unescaped_text)
 
         # Detect escaping at the start and end of a string (we actually care only for
         # whitespace escapes, but detecting more does not matter)
-        if text and text[0] != unescaped_text[0]:
+        if text and unescaped_text and text[0] != unescaped_text[0]:
             cleanup_start = False
-        if text and text[-1] != unescaped_text[-1]:
+        if text and unescaped_text and text[-1] != unescaped_text[-1]:
             cleanup_end = False
 
         return unescaped_text, cleanup_start, cleanup_end
@@ -145,7 +165,7 @@ class DecodingXMLParser:
             return text.lstrip()
         return text
 
-    def emit(self):
+    def emit(self) -> None:
         if self.emit_start is not None:
             text = self.text[self.emit_start : self.parser.CurrentByteIndex].decode(
                 "utf-8"
@@ -161,34 +181,34 @@ class DecodingXMLParser:
         self.raw_string = True
         self.in_string = False
 
-    def StartElementHandler(self, _name, _attributes):
+    def StartElementHandler(self, _name, _attributes) -> None:
         self.emit()
         if self.depth >= self.EMIT_DEPTH:
             self.emit_start = self.parser.CurrentByteIndex
         self.depth += 1
 
-    def EndElementHandler(self, _name):
+    def EndElementHandler(self, _name) -> None:
         self.emit()
         if self.depth >= self.EMIT_DEPTH:
             self.emit_start = self.parser.CurrentByteIndex
         self.depth -= 1
 
-    def CharacterDataHandler(self, _data):
+    def CharacterDataHandler(self, _data) -> None:
         if not self.character_data and not self.in_string:
             self.emit()
             self.emit_start = self.parser.CurrentByteIndex
             self.raw_string = False
             self.in_string = True
 
-    def StartCdataSectionHandler(self):
+    def StartCdataSectionHandler(self) -> None:
         self.emit()
         self.character_data = True
         self.emit_start = self.parser.CurrentByteIndex
 
-    def EndCdataSectionHandler(self):
+    def EndCdataSectionHandler(self) -> None:
         self.character_data = False
 
-    def DefaultHandler(self, _data):
+    def DefaultHandler(self, _data) -> None:
         if self.depth >= self.EMIT_DEPTH:
             self.emit()
             self.emit_start = self.parser.CurrentByteIndex
@@ -205,10 +225,9 @@ class DecodingXMLParser:
 
 class EncodingXMLParser(DecodingXMLParser):
     EMIT_DEPTH = 0
-    FOREGIN_DTD = False
+    FOREIGN_DTD = False
 
-    @staticmethod
-    def process_string(text: str) -> tuple[str, bool, bool]:
+    def process_string(self, text: str) -> tuple[str, bool, bool]:
         return (
             AndroidResourceUnit.escape(text, quote_wrapping_whitespaces=False),
             False,
@@ -225,6 +244,16 @@ class AndroidResourceUnit(base.TranslationUnit):
 
     SINGULAR_TAG = "string"
     PLURAL_TAG = "plurals"
+    ESCAPE_TRANSLATE: ClassVar = str.maketrans(
+        {
+            "\\": "\\\\",
+            "\n": "\\n",
+            "\t": "\\t",
+            "'": "\\'",
+            '"': '\\"',
+        }
+    )
+    ESCAPE_ALL: ClassVar[bool] = True
 
     @classmethod
     def createfromxmlElement(cls, element):
@@ -234,7 +263,7 @@ class AndroidResourceUnit(base.TranslationUnit):
             term = cls(None, xmlelement=element)
         return term
 
-    def __init__(self, source, empty=False, xmlelement=None, **kwargs):
+    def __init__(self, source, empty=False, xmlelement=None, **kwargs) -> None:
         if xmlelement is not None:
             self.xmlelement = xmlelement
         elif self.hasplurals(source):
@@ -251,7 +280,7 @@ class AndroidResourceUnit(base.TranslationUnit):
     def marktranslatable(self, value: bool) -> None:
         return self.xmlelement.set("translatable", "true" if value else "false")
 
-    def isblank(self):
+    def isblank(self) -> bool:
         return not bool(self.getid())
 
     def getid(self):
@@ -267,10 +296,14 @@ class AndroidResourceUnit(base.TranslationUnit):
     def xml_escape_space(matchobj):
         return matchobj.group(0).replace("  ", r" \u0020")
 
+    @overload
     @classmethod
-    def escape(
-        cls, text: str | None, quote_wrapping_whitespaces: bool = True
-    ) -> str | None:
+    def escape(cls, text: str, quote_wrapping_whitespaces: bool = True) -> str: ...
+    @overload
+    @classmethod
+    def escape(cls, text: None, quote_wrapping_whitespaces: bool = True) -> None: ...
+    @classmethod
+    def escape(cls, text, quote_wrapping_whitespaces=True):
         """
         Escape all the characters which need to be escaped in an Android XML
         file.
@@ -286,7 +319,7 @@ class AndroidResourceUnit(base.TranslationUnit):
             return ""
 
         # Escape XML chars and whitespace
-        text = text.translate(ESCAPE_TRANSLATE)
+        text = text.translate(cls.ESCAPE_TRANSLATE)
 
         # @ and ? needs to be escaped at start as this would be interpreted
         # as string/style references
@@ -313,32 +346,31 @@ class AndroidResourceUnit(base.TranslationUnit):
 
     def get_xml_text_value(self, xmltarget):
         raw_xml = etree.tostring(xmltarget, encoding="unicode", with_tail=False)
+        # Use decoding parser to keep XML entities, CDATA while processing Android escaping
+        parser = DecodingXMLParser(raw_xml, escape_all=self.ESCAPE_ALL)
+
         # Unescape as plain text in case there is no XML markup. Ufortunately
-        # CDATA elements are not listed as childs in lxml, so test for it in raw XML.
+        # CDATA elements are not listed as children in lxml, so test for it in raw XML.
         if len(xmltarget) == 0 and "<![CDATA[" not in raw_xml:
             if xmltarget.text is None:
                 return ""
-            text, cleanup_start, cleanup_end = DecodingXMLParser.process_string(
-                xmltarget.text
-            )
+            text, cleanup_start, cleanup_end = parser.process_string(xmltarget.text)
             if cleanup_start:
                 text = text.lstrip()
             if cleanup_end:
                 text = text.rstrip()
             return text
 
-        # Use decoding parser to keep XML entitities, CDATA while processing Android escaping
-        parser = DecodingXMLParser(raw_xml)
         return parser.parse()
 
-    def set_xml_text_plain(self, target, xmltarget):
+    def set_xml_text_plain(self, target, xmltarget) -> None:
         # Remove possible old elements
         for x in xmltarget.iterchildren():
             xmltarget.remove(x)
         # Handle text only
         xmltarget.text = self.escape(target)
 
-    def set_xml_text_value(self, target, xmltarget):
+    def set_xml_text_value(self, target, xmltarget) -> None:
         if "<" in target or "&" in target:
             # Try to handle it as legacy XML
             parser = etree.XMLParser(strip_cdata=False, resolve_entities=False)
@@ -387,15 +419,6 @@ class AndroidResourceUnit(base.TranslationUnit):
             return "en"
         return super().gettargetlanguage()
 
-    def get_plural_tags(self):
-        locale = self.gettargetlanguage()
-        if not locale:
-            return data.plural_tags["en"]
-        # Handle b+ style language codes
-        locale = locale.removeprefix("b+")
-        locale = locale.replace("_", "-").replace("+", "-").split("-")[0]
-        return data.plural_tags.get(locale, data.plural_tags["en"])
-
     @property
     def target(self):
         if self.xmlelement.tag != self.PLURAL_TAG:
@@ -404,19 +427,25 @@ class AndroidResourceUnit(base.TranslationUnit):
             entry.get("quantity"): self.get_xml_text_value(entry)
             for entry in self.xmlelement.iterchildren("item")
         }
-        plural_tags = self.get_plural_tags()
+        plural_tags = (
+            self._store.get_plural_tags() if self._store else data.plural_tags["en"]
+        )
         return multistring([plurals.get(tag, "") for tag in plural_tags])
 
     @target.setter
-    def target(self, target):
+    def target(self, target) -> None:
         if self.hasplurals(self.source) or self.hasplurals(target):
             # Fix the root tag if mismatching
             if self.xmlelement.tag != self.PLURAL_TAG:
                 old_id = self.getid()
-                self.xmlelement = etree.Element(self.PLURAL_TAG)
+                newelement = etree.Element(self.PLURAL_TAG)
+                parent = self.xmlelement.getparent()
+                if parent is not None:
+                    parent.replace(self.xmlelement, newelement)
+                self.xmlelement = newelement
                 self.setid(old_id)
 
-            plural_tags = self.get_plural_tags()
+            plural_tags = self._store.get_plural_tags()
 
             # Sync plural_strings elements to plural_tags count.
             plural_strings = self.sync_plural_count(target, plural_tags)
@@ -427,14 +456,28 @@ class AndroidResourceUnit(base.TranslationUnit):
 
             self.xmlelement.text = "\n    "
 
+            # Include additional plural for decimal numbers if not present. This is
+            # enforced by Android lint but translate-toolkit currently does not support
+            # editing this.
+            locale = self._store.get_base_locale_code()
+            if locale in data.DECIMAL_EXTRA_TAGS:
+                for extra in data.DECIMAL_EXTRA_TAGS[locale]:
+                    if extra not in plural_tags:
+                        # Create copy here to avoid modifications to language.data
+                        plural_tags = [*plural_tags, extra]
+                        plural_strings.append(plural_strings[-1])
+
             # Include "other" as copy of "many" if "other" is not present. This avoids crashes
             # of Android builts with broken plurals handling.
             if "other" not in plural_tags and "many" in plural_tags:
-                # Create copy here to avoid modifications to laguage.data
+                # Create copy here to avoid modifications to language.data
                 plural_tags = [*plural_tags, "other"]
                 plural_strings.append(plural_strings[-1])
 
-            for plural_tag, plural_string in zip(plural_tags, plural_strings):
+            for plural_tag, plural_string in sorted(
+                zip(plural_tags, plural_strings, strict=True),
+                key=lambda item: data.cldr_plural_categories.index(item[0]),
+            ):
                 item = etree.Element("item")
                 item.set("quantity", plural_tag)
                 self.xmlelement.append(item)
@@ -443,7 +486,11 @@ class AndroidResourceUnit(base.TranslationUnit):
             # Fix the root tag if mismatching
             if self.xmlelement.tag != self.SINGULAR_TAG:
                 old_id = self.getid()
-                self.xmlelement = etree.Element(self.SINGULAR_TAG)
+                newelement = etree.Element(self.SINGULAR_TAG)
+                parent = self.xmlelement.getparent()
+                if parent is not None:
+                    parent.replace(self.xmlelement, newelement)
+                self.xmlelement = newelement
                 self.setid(old_id)
 
             self.set_xml_text_value(target, self.xmlelement)
@@ -455,7 +502,7 @@ class AndroidResourceUnit(base.TranslationUnit):
         return self.xmlelement
 
     # Notes are handled as previous sibling comments.
-    def addnote(self, text, origin=None, position="append"):
+    def addnote(self, text, origin=None, position="append") -> None:
         if origin in {"programmer", "developer", "source code", None}:
             self.xmlelement.addprevious(etree.Comment(text))
         else:
@@ -473,7 +520,7 @@ class AndroidResourceUnit(base.TranslationUnit):
             return "\n".join(comments)
         return super().getnotes(origin)
 
-    def removenotes(self, origin=None):
+    def removenotes(self, origin=None) -> None:
         if (self.xmlelement is not None) and (self.xmlelement.getparent is not None):
             prevSibling = self.xmlelement.getprevious()
             while (prevSibling is not None) and (prevSibling.tag is etree.Comment):
@@ -482,7 +529,7 @@ class AndroidResourceUnit(base.TranslationUnit):
 
         super().removenotes()
 
-    def __str__(self):
+    def __str__(self) -> str:
         return etree.tostring(self.xmlelement, pretty_print=True, encoding="unicode")
 
     def __eq__(self, other):
@@ -504,10 +551,10 @@ class AndroidResourceFile(lisa.LISAfile):
     bodyNode = "resources"
     XMLskeleton = """<?xml version="1.0" encoding="utf-8"?>
 <resources></resources>"""
-    XMLindent = {"indent": "    ", "leaves": {"string", "item"}}
+    XMLindent = {"indent": "    ", "leaves": {"string", "item"}, "trailing_eol": True}
     XMLuppercaseEncoding = False
 
-    def initbody(self):
+    def initbody(self) -> None:
         """
         Initialises self.body so it never needs to be retrieved from the XML
         again.
@@ -515,7 +562,7 @@ class AndroidResourceFile(lisa.LISAfile):
         self.namespace = self.document.getroot().nsmap.get(None, None)
         self.body = self.document.getroot()
 
-    def parse(self, xml):
+    def parse(self, xml) -> None:
         """Populates this object from the given xml string."""
         if not hasattr(self, "filename"):
             self.filename = getattr(xml, "name", "")
@@ -553,7 +600,7 @@ class AndroidResourceFile(lisa.LISAfile):
 
         return target_lang
 
-    def addunit(self, unit, new=True):
+    def addunit(self, unit, new=True) -> None:
         """
         Adds unit to the document.
 
@@ -591,14 +638,33 @@ class AndroidResourceFile(lisa.LISAfile):
         if do_cleanup:
             etree.cleanup_namespaces(self.body, top_nsmap=newns)
 
-    def removeunit(self, unit):
+    def removeunit(self, unit) -> None:
         unit.removenotes()
         super().removeunit(unit)
 
 
-class MOKOResourceUnit(AndroidResourceUnit):
+class CMPResourceUnit(AndroidResourceUnit):
+    # Only \n and \t are escaped here (+ Unicode escape sequences), see
+    # https://github.com/JetBrains/compose-multiplatform/blob/754dc253964e5aeab63581ed2a6a4ac376af1c32/gradle-plugins/compose/src/main/kotlin/org/jetbrains/compose/resources/PrepareComposeResources.kt#L270-L298
+    ESCAPE_TRANSLATE = str.maketrans(
+        {
+            "\\": "\\\\",
+            "\n": "\\n",
+            "\t": "\\t",
+        }
+    )
+    ESCAPE_ALL: ClassVar[bool] = False
+
+
+class CMPResourceFile(AndroidResourceFile):
+    UnitClass = CMPResourceUnit
+    Name = "Compose Multiplatform Resources"
+
+
+class MOKOResourceUnit(CMPResourceUnit):
     PLURAL_TAG = "plural"
 
 
 class MOKOResourceFile(AndroidResourceFile):
     UnitClass = MOKOResourceUnit
+    Name = "Mobile Kotlin Resources"
