@@ -24,7 +24,7 @@ import codecs
 import logging
 from io import BytesIO
 from itertools import starmap
-from typing import TYPE_CHECKING, ClassVar, Literal
+from typing import TYPE_CHECKING, Any, ClassVar, Literal
 
 from translate.lang.data import plural_tags
 from translate.misc.multistring import multistring
@@ -52,10 +52,10 @@ MISSING = object()
 
 
 class ParseError(Exception):
-    def __init__(self, inner_exc):
+    def __init__(self, inner_exc) -> None:
         self.inner_exc = inner_exc
 
-    def __str__(self):
+    def __str__(self) -> str:
         return repr(self.inner_exc)
 
 
@@ -84,7 +84,7 @@ class TranslationUnit:
 
     """
 
-    rich_parsers: list[Callable[[str], list[StringElem]]] = []
+    rich_parsers: list[Callable[[str], list[StringElem] | None]] = []
     """A list of functions to use for parsing a string into a rich string
     tree."""
 
@@ -127,20 +127,29 @@ class TranslationUnit:
     _rich_source = None
     _rich_target = None
     _state_n = 0
+    _line_number: int | None = None
     notes = ""
 
-    def __init__(self, source=None):
+    def __init__(self, source=None) -> None:
         """Constructs a TranslationUnit containing the given source string."""
         if source is not None:
             self.source = source
 
-    def __eq__(self, other):
+    @property
+    def line_number(self) -> int | None:
+        """
+        Line number in the source file where this unit was found.
+
+        The line number is 1-based (first line is line 1). Returns None if the
+        format doesn't support line numbering or if the information is not available.
+        """
+        return self._line_number
+
+    def __eq__(self, other: TranslationUnit) -> bool:
         """
         Compares two TranslationUnits.
 
-        :type other: :class:`TranslationUnit`
         :param other: Another :class:`TranslationUnit`
-        :rtype: Boolean
         :return: Returns *True* if the supplied :class:`TranslationUnit`
                  equals this unit.
         """
@@ -156,7 +165,7 @@ class TranslationUnit:
     def __hash__(self):
         return hash((self.source, self.target, self.getid()))
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Converts to a string representation. Most often overridden by subclasses."""
         # no point in showing store object.
         return ", ".join(
@@ -200,7 +209,7 @@ class TranslationUnit:
         return self._source
 
     @source.setter
-    def source(self, source):
+    def source(self, source) -> None:
         """Set the source string to the given value."""
         self._rich_source = None
         self._source = source
@@ -210,7 +219,7 @@ class TranslationUnit:
         return self._target
 
     @target.setter
-    def target(self, target):
+    def target(self, target) -> None:
         """Set the target string to the given value."""
         self._rich_target = None
         self._target = target
@@ -223,7 +232,7 @@ class TranslationUnit:
         return self._rich_source
 
     @rich_source.setter
-    def rich_source(self, value):
+    def rich_source(self, value) -> None:
         if not hasattr(value, "__iter__"):
             raise ValueError("value must be iterable")
         if len(value) < 1:
@@ -243,7 +252,7 @@ class TranslationUnit:
         return self._rich_target
 
     @rich_target.setter
-    def rich_target(self, value):
+    def rich_target(self, value) -> None:
         if not hasattr(value, "__iter__"):
             raise ValueError("value must be iterable")
         if len(value) < 1:
@@ -253,12 +262,10 @@ class TranslationUnit:
         self._rich_target = list(value)
         self.target = self.rich_to_multistring(value)
 
-    def gettargetlen(self):
+    def gettargetlen(self) -> int:
         """
         Returns the length of the target string.
 
-        :rtype: Integer
-
         .. note::
 
            Plural forms might be combined.
@@ -269,11 +276,10 @@ class TranslationUnit:
             length += sum(len(pluralform) for pluralform in strings[1:])
         return length
 
-    def getid(self):
+    def getid(self) -> str:
         """
         A unique identifier for this unit.
 
-        :rtype: string
         :return: an identifier for this unit that is unique in the store
 
         Derived classes should override this in a way that guarantees a unique
@@ -281,7 +287,7 @@ class TranslationUnit:
         """
         return self.source
 
-    def setid(self, value):
+    def setid(self, value) -> None:
         """
         Sets the unique identified for this unit.
 
@@ -290,19 +296,17 @@ class TranslationUnit:
         """
 
     @staticmethod
-    def getlocations():
+    def getlocations() -> list[str]:
         """
         A list of source code locations.
 
-        :rtype: List
-
         .. note::
 
            Shouldn't be implemented if the format doesn't support it.
         """
         return []
 
-    def addlocation(self, location):
+    def addlocation(self, location) -> None:
         """
         Add one location to the list of locations.
 
@@ -311,7 +315,7 @@ class TranslationUnit:
            Shouldn't be implemented if the format doesn't support it.
         """
 
-    def addlocations(self, location):
+    def addlocations(self, location) -> None:
         """
         Add a location or a list of locations.
 
@@ -330,11 +334,11 @@ class TranslationUnit:
         else:
             self.addlocation(location)
 
-    def getcontext(self):
+    def getcontext(self) -> str:
         """Get the message context."""
         return ""
 
-    def setcontext(self, context):
+    def setcontext(self, context) -> None:
         """Set the message context."""
 
     def getnotes(self, origin=None):
@@ -348,13 +352,16 @@ class TranslationUnit:
         """
         return getattr(self, "notes", "")
 
-    def addnote(self, text, origin=None, position="append"):
+    def addnote(
+        self,
+        text: str,
+        origin: str | None = None,
+        position: Literal["append", "replace", "merge"] = "append",
+    ) -> None:
         """
         Adds a note (comment).
 
-        :type text: string
         :param text: Usually just a sentence or two.
-        :type origin: string
         :param origin: Specifies who/where the comment comes from.
                        Origin can be one of the following text strings:
                        - 'translator'
@@ -365,30 +372,24 @@ class TranslationUnit:
         else:
             self.notes = text
 
-    def removenotes(self, origin=None):
+    def removenotes(self, origin=None) -> None:
         """Remove all the translator's notes."""
         self.notes = ""
 
-    def adderror(self, errorname, errortext):
+    def adderror(self, errorname: str, errortext: str) -> None:
         """
         Adds an error message to this unit.
 
-        :type errorname: string
         :param errorname: A single word to id the error.
-        :type errortext: string
         :param errortext: The text describing the error.
         """
 
     @staticmethod
-    def geterrors():
-        """
-        Get all error messages.
-
-        :rtype: Dictionary
-        """
+    def geterrors() -> dict[str, str]:
+        """Get all error messages."""
         return {}
 
-    def markreviewneeded(self, needsreview=True, explanation=None):
+    def markreviewneeded(self, needsreview=True, explanation=None) -> None:
         """
         Marks the unit to indicate whether it needs review.
 
@@ -419,19 +420,19 @@ class TranslationUnit:
         """Marks the unit as translatable or not."""
 
     @staticmethod
-    def isfuzzy():
+    def isfuzzy() -> bool:
         """Indicates whether this unit is fuzzy."""
         return False
 
-    def markfuzzy(self, value=True):
+    def markfuzzy(self, value=True) -> None:
         """Marks the unit as fuzzy or not."""
 
     @staticmethod
-    def isobsolete():
+    def isobsolete() -> bool:
         """Indicate whether a unit is obsolete."""
         return False
 
-    def makeobsolete(self):
+    def makeobsolete(self) -> None:
         """Make a unit obsolete."""
 
     @staticmethod
@@ -457,7 +458,7 @@ class TranslationUnit:
         return not (self.source or self.target)
 
     @staticmethod
-    def hasplural():
+    def hasplural() -> bool:
         """Tells whether or not this specific unit has plural strings."""
         # TODO: Reconsider
         return False
@@ -468,7 +469,9 @@ class TranslationUnit:
     def gettargetlanguage(self):
         return self._store.gettargetlanguage()
 
-    def merge(self, otherunit, overwrite=False, comments=True, authoritative=False):
+    def merge(
+        self, otherunit, overwrite=False, comments=True, authoritative=False
+    ) -> None:
         """Do basic format agnostic merging."""
         if not self.target or overwrite:
             self.rich_target = otherunit.rich_target
@@ -520,10 +523,10 @@ class TranslationUnit:
             return self._state_n
         return self.S_UNREVIEWED if self.istranslated() else self.S_EMPTY
 
-    def set_state_n(self, value):
+    def set_state_n(self, value) -> None:
         self._state_n = value
 
-    def infer_state(self):
+    def infer_state(self) -> None:
         """
         Empty method that should be overridden in sub-classes to infer the
         current state(_n) of the unit from its current state.
@@ -552,6 +555,93 @@ class TranslationUnit:
         return plural_strings[: len(plural_tags)]
 
 
+class MetadataTranslationUnit(TranslationUnit):
+    """
+    Base class for translation units that store field data in an internal dictionary.
+
+    This class provides a common implementation for storage formats (catkeys, omegat,
+    utx, wordfast, ARB, RESJSON) that manage unit data through an internal dictionary
+    accessible via a `metadata` property with getters and setters.
+    """
+
+    def __init__(self, *args, metadata=None, **kwargs):
+        """
+        Initialize the internal dictionary.
+
+        Note: _metadata_dict is initialized before calling super().__init__() because
+        the parent class (TranslationUnit) may set properties (like source)
+        that depend on the dictionary being available.
+
+        :param metadata: Optional initial metadata dictionary
+        """
+        self._metadata_dict: dict[str, Any] = metadata or {}
+        super().__init__(*args, **kwargs)
+
+    def getmetadata(self) -> dict[str, Any]:
+        """
+        Get the dictionary of metadata/field values for this unit.
+
+        :return: The internal dictionary containing field values
+        """
+        return self._metadata_dict
+
+    def setmetadata(self, newdict: dict[str, Any]) -> None:
+        """
+        Set the dictionary of metadata/field values for this unit.
+
+        :param newdict: A new dictionary with field values
+        """
+        self._metadata_dict = newdict
+
+    metadata = property(getmetadata, setmetadata)
+
+    def __eq__(self, other: TranslationUnit) -> bool:
+        """
+        Compare two units including their metadata.
+
+        Metadata dictionaries are compared by considering only non-empty values
+        to handle cases where parsed units have all fields (including empty strings)
+        while programmatically created units only have set fields.
+
+        :param other: Another TranslationUnit
+        :return: True if units are equal including metadata
+        """
+        # First check the base equality (source, target, id)
+        if not super().__eq__(other):
+            return False
+
+        # Then check metadata if the other unit has it
+        if isinstance(other, MetadataTranslationUnit):
+            # Filter out empty string values for comparison
+            # This handles the case where parsed units have all fields (with empty strings)
+            # while created units only have the fields that were set
+            def filter_empty(d):
+                return {k: v for k, v in d.items() if v}
+
+            return filter_empty(self._metadata_dict) == filter_empty(
+                other._metadata_dict
+            )
+
+        return True
+
+    def __hash__(self):
+        """
+        Generate hash including metadata.
+
+        Only non-empty metadata values are included in the hash to match
+        the equality behavior.
+
+        :return: Hash value for the unit
+        """
+        # Hash the base attributes plus the non-empty metadata dict items
+        base_hash = super().__hash__()
+        # Filter out empty values to match __eq__ behavior
+        metadata_items = tuple(
+            sorted((k, v) for k, v in self._metadata_dict.items() if v)
+        )
+        return hash((base_hash, metadata_items))
+
+
 class TranslationStore:
     """Base class for stores for multiple translation units of type UnitClass."""
 
@@ -573,7 +663,7 @@ class TranslationStore:
     sourcelanguage = None
     targetlanguage = None
 
-    def __init__(self, unitclass=None, encoding=None):
+    def __init__(self, unitclass=None, encoding=None) -> None:
         """Construct a blank TranslationStore."""
         self.units = []
         if unitclass:
@@ -591,7 +681,7 @@ class TranslationStore:
         return self._encoding or self.default_encoding
 
     @encoding.setter
-    def encoding(self, value):
+    def encoding(self, value) -> None:
         if value == "CHARSET" or value is None:
             return
         if value == "ascii":
@@ -602,7 +692,7 @@ class TranslationStore:
         """Get the source language for this store."""
         return self.sourcelanguage
 
-    def setsourcelanguage(self, sourcelanguage):
+    def setsourcelanguage(self, sourcelanguage) -> None:
         """Set the source language for this store."""
         self.sourcelanguage = sourcelanguage
 
@@ -610,7 +700,7 @@ class TranslationStore:
         """Get the target language for this store."""
         return self.targetlanguage
 
-    def settargetlanguage(self, targetlanguage):
+    def settargetlanguage(self, targetlanguage) -> None:
         """Set the target language for this store."""
         self.targetlanguage = targetlanguage
 
@@ -618,7 +708,7 @@ class TranslationStore:
         """Get the project type for this store."""
         return getattr(self, "_project_style", None)
 
-    def setprojectstyle(self, project_style):
+    def setprojectstyle(self, project_style) -> None:
         """Set the project type for this store."""
         self._project_style = project_style
 
@@ -630,38 +720,32 @@ class TranslationStore:
         """Return a list of all units in this store."""
         return list(self.unit_iter())
 
-    def addunit(self, unit):
+    def addunit(self, unit: TranslationUnit) -> None:
         """
         Append the given unit to the object's list of units.
 
         This method should always be used rather than trying to modify the
         list manually.
 
-        :type unit: :class:`TranslationUnit`
         :param unit: The unit that will be added.
         """
         unit._store = self
         self.units.append(unit)
 
-    def removeunit(self, unit):
+    def removeunit(self, unit: TranslationUnit) -> None:
         """
         Remove the given unit to the object's list of units.
 
         This method should always be used rather than trying to modify the
         list manually.
 
-        :type unit: :class:`TranslationUnit`
         :param unit: The unit that will be added.
         """
         self.units.remove(unit)
         self.remove_unit_from_index(unit)
 
-    def addsourceunit(self, source):
-        """
-        Add and returns a new unit with the given source string.
-
-        :rtype: :class:`TranslationUnit`
-        """
+    def addsourceunit(self, source: str) -> TranslationUnit:
+        """Add and returns a new unit with the given source string."""
         unit = self.UnitClass(source)
         self.addunit(unit)
         return unit
@@ -671,43 +755,31 @@ class TranslationStore:
         self.require_index()
         return self.id_index.get(id)
 
-    def findunit(self, source):
-        """
-        Find the unit with the given source string.
-
-        :rtype: :class:`TranslationUnit` or None
-        """
+    def findunit(self, source: str) -> TranslationUnit | None:
+        """Find the unit with the given source string."""
         self.require_index()
         if source in self.sourceindex:
             return self.sourceindex[source][0]
         return None
 
-    def findunits(self, source):
-        """
-        Find the units with the given source string.
-
-        :rtype: :class:`TranslationUnit` or None
-        """
+    def findunits(self, source: str) -> list[TranslationUnit] | None:
+        """Find the units with the given source string."""
         self.require_index()
         if source in self.sourceindex:
             return self.sourceindex[source]
         return None
 
-    def translate(self, source):
-        """
-        Return the translated string for a given source string.
-
-        :rtype: String or None
-        """
+    def translate(self, source: str) -> str | None:
+        """Return the translated string for a given source string."""
         unit = self.findunit(source)
         if unit and unit.target:
             return unit.target
         return None
 
-    def remove_unit_from_index(self, unit):
+    def remove_unit_from_index(self, unit) -> None:
         """Remove a unit from source and locaton indexes."""
 
-        def remove_source(source):
+        def remove_source(source) -> None:
             if source in self.sourceindex:
                 self.sourceindex[source].remove(unit)
                 if len(self.sourceindex[source]) == 0:
@@ -723,11 +795,11 @@ class TranslationStore:
             if location in self.locationindex and self.locationindex[location] == unit:
                 del self.locationindex[location]
 
-    def add_unit_to_index(self, unit):
+    def add_unit_to_index(self, unit) -> None:
         """Add a unit to source and location indices."""
         self.id_index[unit.getid()] = unit
 
-        def insert_unit(source):
+        def insert_unit(source) -> None:
             if source not in self.sourceindex:
                 self.sourceindex[source] = [unit]
             else:
@@ -746,7 +818,7 @@ class TranslationStore:
                 # case there are several units with the same location.
                 self.locationindex[location] = unit
 
-    def makeindex(self):
+    def makeindex(self) -> None:
         """
         Indexes the items in this store. At least .sourceindex should be
         useful.
@@ -759,7 +831,7 @@ class TranslationStore:
             if not (unit.isheader() or unit.isblank()):
                 self.add_unit_to_index(unit)
 
-    def require_index(self):
+    def require_index(self) -> None:
         """Make sure source index exists."""
         if not self.id_index:
             self.makeindex()
@@ -775,7 +847,7 @@ class TranslationStore:
         odict["fileobj"] = None
         return odict
 
-    def __bytes__(self):
+    def __bytes__(self) -> bytes:
         out = BytesIO()
         self.serialize(out)
         return out.getvalue()
@@ -802,7 +874,7 @@ class TranslationStore:
             return True
         return all(not unit.istranslatable() for unit in self.units)
 
-    def _assignname(self):
+    def _assignname(self) -> None:
         """
         Tries to work out what the name of the filesystem file is and
         assigns it to .filename.
@@ -916,7 +988,7 @@ class TranslationStore:
             "Subclasses must provide their own parsing method."
         )
 
-    def savefile(self, storefile):
+    def savefile(self, storefile) -> None:
         """Write the string representation to the given file (or filename)."""
         if isinstance(storefile, str):
             storefile = open(storefile, "wb")
@@ -925,7 +997,7 @@ class TranslationStore:
         self.serialize(storefile)
         storefile.close()
 
-    def save(self):
+    def save(self) -> None:
         """Save to the file that data was originally read from, if available."""
         fileobj = getattr(self, "fileobj", None)
         if not fileobj:
@@ -969,12 +1041,11 @@ class TranslationStore:
         return newstore
 
     @property
-    def merge_on(self):
+    def merge_on(self) -> str:
         """
         The matching criterion to use when merging on.
 
         :return: The default matching criterion for all the subclasses.
-        :rtype: string
         """
         return "id"
 
@@ -996,11 +1067,11 @@ class UnitId:
     KEY_SEPARATOR = "."
     INDEX_SEPARATOR = ""
 
-    def __init__(self, parts):
+    def __init__(self, parts) -> None:
         self.parts = parts
 
-    def __str__(self):
-        def fmt(element, key):
+    def __str__(self) -> str:
+        def fmt(element, key) -> str:
             if element == "key":
                 return f"{self.KEY_SEPARATOR}{key}"
             if element == "index":
@@ -1045,14 +1116,14 @@ class UnitId:
     def __eq__(self, other):
         return self.parts == other.parts
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"<UnitId:{self.parts}>"
 
 
 class DictUnit(TranslationUnit):
     IdClass = UnitId
 
-    def __init__(self, source=None):
+    def __init__(self, source=None) -> None:
         super().__init__(source)
         self._unitid = None
 
@@ -1061,7 +1132,7 @@ class DictUnit(TranslationUnit):
             self._unitid = self.IdClass.from_string(self._id)
         return self._unitid
 
-    def storevalue(self, output, value, override_key=None, unset=False):
+    def storevalue(self, output, value, override_key=None, unset=False) -> None:
         if isinstance(value, multistring):
             value = str(value)
         parent = target = output
@@ -1116,7 +1187,7 @@ class DictUnit(TranslationUnit):
         else:
             raise ValueError(f"Unsupported element: {child_element}")
 
-    def storevalues(self, output):
+    def storevalues(self, output) -> None:
         self.storevalue(output, self.value)
 
     def getvalue(self):
@@ -1125,11 +1196,11 @@ class DictUnit(TranslationUnit):
         self.storevalues(result)
         return result
 
-    def setid(self, value, unitid=None):
+    def setid(self, value, unitid=None) -> None:
         self._id = value
         self._unitid = unitid
 
-    def set_unitid(self, unitid):
+    def set_unitid(self, unitid) -> None:
         # Set _unitid first to avoid need to re-construct it from id
         self.setid(str(unitid), unitid)
 
@@ -1142,6 +1213,6 @@ class DictStore(TranslationStore):
             return []
         return {}
 
-    def serialize_units(self, output):
+    def serialize_units(self, output) -> None:
         for unit in self.unit_iter():
             unit.storevalues(output)
