@@ -59,11 +59,11 @@ if TYPE_CHECKING:
 class MarkdownUnit(base.TranslationUnit):
     """A unit of translatable/localisable markdown content."""
 
-    def __init__(self, source=None):
+    def __init__(self, source=None) -> None:
         super().__init__(source)
         self.locations = []
 
-    def addlocation(self, location):
+    def addlocation(self, location) -> None:
         self.locations.append(location)
 
     def getlocations(self):
@@ -72,14 +72,14 @@ class MarkdownUnit(base.TranslationUnit):
 
 class MarkdownFrontmatterUnit(MarkdownUnit):
     @staticmethod
-    def isheader():
+    def isheader() -> bool:
         return True
 
 
 class MarkdownFile(base.TranslationStore):
     UnitClass = MarkdownUnit
 
-    def __init__(self, inputfile=None, callback=None, max_line_length=None):
+    def __init__(self, inputfile=None, callback=None, max_line_length=None) -> None:
         """
         Construct a new object instance.
 
@@ -100,7 +100,7 @@ class MarkdownFile(base.TranslationStore):
             inputfile.close()
             self.parse(md_src)
 
-    def parse(self, data):
+    def parse(self, data) -> None:
         """Process the given source string (binary)."""
         lines = data.decode().splitlines(keepends=False)
         front_matter_end = 0
@@ -156,11 +156,14 @@ class MarkdownFile(base.TranslationStore):
 
 
 class TranslatingMarkdownRenderer(MarkdownRenderer):
-    def __init__(self, translate_callback, *extras, max_line_length: int | None = None):
+    def __init__(
+        self, translate_callback, *extras, max_line_length: int | None = None
+    ) -> None:
         super().__init__(*extras, max_line_length=max_line_length)
         self.translate_callback = translate_callback
         self.bypass = False
         self.path = []
+        self.ignore_translation = False
 
     def render(self, token: mistletoe.token.Token) -> str:
         try:
@@ -224,7 +227,7 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
 
         if token.dest_type in {"uri", "angle_uri"}:
             # Markdown link format: "[" description "](" dest_part [" " title] ")"
-            dest_part = "<" + target + ">" if token.dest_type == "angle_uri" else target
+            dest_part = f"<{target}>" if token.dest_type == "angle_uri" else target
             placeholder = Fragment(None, important=True)
             placeholder.placeholder_content = [
                 Fragment("("),
@@ -271,27 +274,33 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
         self, token: LinkReferenceDefinition
     ) -> Iterable[Fragment]:
         # note: these tokens will never be encountered in bypass mode.
-        translated_label = self.translate_callback(
-            token.label, [*self.path, "link-label"]
-        )
+        # Translate label and title unless we're in an ignore section
+        if self.ignore_translation:
+            label = token.label
+            title = token.title
+        else:
+            label = self.translate_callback(token.label, [*self.path, "link-label"])
+            title = (
+                self.translate_callback(token.title, [*self.path, "link-title"])
+                if token.title
+                else None
+            )
+
         placeholder = Fragment(None)
         placeholder.placeholder_content = [
             Fragment("["),
-            Fragment(translated_label, wordwrap=True),
+            Fragment(label, wordwrap=True),
             Fragment("]: ", wordwrap=True),
             Fragment(
-                "<" + token.dest + ">" if token.dest_type == "angle_uri" else token.dest
+                f"<{token.dest}>" if token.dest_type == "angle_uri" else token.dest
             ),
         ]
-        if token.title:
-            translated_title = self.translate_callback(
-                token.title, [*self.path, "link-title"]
-            )
+        if title:
             placeholder.placeholder_content.extend(
                 [
                     Fragment(" ", wordwrap=True),
                     Fragment(token.title_delimiter),
-                    Fragment(translated_title, wordwrap=True),
+                    Fragment(title, wordwrap=True),
                     Fragment(
                         ")" if token.title_delimiter == "(" else token.title_delimiter
                     ),
@@ -336,6 +345,19 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
         self.path.pop()
         return content
 
+    def render_html_block(
+        self, token: block_token.HtmlBlock, max_line_length: int
+    ) -> Iterable[str]:
+        # Check if this is a translation control comment
+        content = token.content.strip()
+        if content == "<!-- translate:off -->":
+            self.ignore_translation = True
+        elif content == "<!-- translate:on -->":
+            self.ignore_translation = False
+
+        # Return the raw HTML block content
+        return super().render_html_block(token, max_line_length=max_line_length)
+
     def render_list_item(
         self, token: block_token.ListItem, max_line_length: int
     ) -> Iterable[str]:
@@ -370,9 +392,24 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
         self, tokens: Iterable[span_token.SpanToken], max_line_length: int
     ) -> Iterable[str]:
         """Renders a sequence of span tokens to markdown, with translation."""
+        # If we're in an ignore section, skip translation
+        if self.ignore_translation:
+            original_bypass = self.bypass
+            try:
+                self.bypass = True
+                fragments = self.make_fragments(tokens)
+                # Expand placeholders before rendering
+                expanded = list(self.expand_placeholders(fragments))
+                return super().fragments_to_lines(
+                    expanded, max_line_length=max_line_length
+                )
+            finally:
+                self.bypass = original_bypass
+
         # turn the span into fragments, which may include placeholders.
         # list-ify the iterator because we may need to traverse it more than once
         fragments = list(self.make_fragments(tokens))
+        original_bypass = self.bypass
         try:
             self.bypass = True
 
@@ -403,7 +440,7 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
             expanded = list(self.expand_placeholders(fragments))
             return super().fragments_to_lines(expanded, max_line_length=max_line_length)
         finally:
-            self.bypass = False
+            self.bypass = original_bypass
 
     @classmethod
     def merge_adjacent_placeholders(
@@ -444,17 +481,17 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
         """
         fragments = list(fragments)
 
-        l = 0
-        while l < len(fragments):
-            if getattr(fragments[l], "placeholder_content", None):
-                if getattr(fragments[l], "important", False):
+        pos = 0
+        while pos < len(fragments):
+            if getattr(fragments[pos], "placeholder_content", None):
+                if getattr(fragments[pos], "important", False):
                     break
-            elif not fragments[l].text.isspace():
+            elif not fragments[pos].text.isspace():
                 break
-            l += 1
+            pos += 1
 
         t = len(fragments)
-        while t - 1 >= l:
+        while t - 1 >= pos:
             if getattr(fragments[t - 1], "placeholder_content", None):
                 if getattr(fragments[t - 1], "important", False):
                     break
@@ -462,7 +499,7 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
                 break
             t -= 1
 
-        return fragments[:l], fragments[l:t], fragments[t:]
+        return fragments[:pos], fragments[pos:t], fragments[t:]
 
     @classmethod
     def insert_placeholder_markers(
@@ -477,7 +514,7 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
             content = getattr(fragment, "placeholder_content", None)
             if content:
                 placeholders.append(fragment)
-                fragment.text = "{%d}" % len(placeholders)
+                fragment.text = f"{{{len(placeholders)}}}"
 
         return placeholders
 
@@ -490,7 +527,7 @@ class TranslatingMarkdownRenderer(MarkdownRenderer):
             content_md = "\n".join(
                 self.fragments_to_lines(content, max_line_length=float("inf"))
             )
-            markdown = markdown.replace("{%d}" % (index + 1), content_md)
+            markdown = markdown.replace(f"{{{index + 1}}}", content_md)
 
         return markdown
 
