@@ -56,13 +56,11 @@ def mergestores(store1, store2, mergeblanks, mergefuzzy, mergecomments):
     return store1
 
 
-def str2bool(option):
+def str2bool(option: str) -> bool:
     """
     Convert a string value to boolean.
 
     :param option: yes, true, 1, no, false, 0
-    :type option: String
-    :rtype: Boolean
 
     """
     option = option.lower()
@@ -80,7 +78,7 @@ def mergestore(
     mergeblanks="no",
     mergefuzzy="no",
     mergecomments="yes",
-):
+) -> int:
     try:
         mergecomments = str2bool(mergecomments)
     except ValueError as error:
@@ -108,7 +106,7 @@ def mergestore(
     return 1
 
 
-def main():
+def main() -> None:
     formats = {
         ("po", "po"): ("po", mergestore),
         ("po", "pot"): ("po", mergestore),
